import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:i_sizing/i_sizing.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:side_inputs/side_inputs.dart';

class DefaultTextFormInput extends HookWidget {
  static const int maxIHeight = 1;
  static const int maxIWidth = 6;
  static const String preferSize = '1x6';
  DefaultTextFormInput({
    Key? key,
    this.prefixIcon,
    this.labelText = '',
    this.hintText = '',
    this.inputKey = '',
    this.onChanged,
    this.validator,
    this.onEditCompleteDone = false,
    this.inputFormatters,
    this.maxLength,
    this.connectionI,
    this.clearIconPressed,
    this.sizeFromParent,
    this.initialValue,
    this.isValid,
    this.autoValidate = false,
    this.textInputKey,
    this.controller,
    this.labelStyle,
    this.border,
    this.enableBorder,
    this.focusBorder,
    this.errorBorder,
    this.suffixStyle,
    this.errorStyle,
    this.helperStyle,
    this.hintStyle,
    this.contentPadding,
    this.style,
    this.prefixStyle,
  }) : super(key: key);

  final String labelText;
  final TextStyle? labelStyle;

  final String inputKey;
  final String? sizeFromParent;
  double? connectionI;
  final String hintText;
  final Widget? prefixIcon;
  void Function(String)? onChanged;
  void Function()? clearIconPressed;
  String? Function(String?)? validator;
  final bool onEditCompleteDone;
  final int? maxLength;
  final String? initialValue;
  final List<TextInputFormatter>? inputFormatters;
  final bool? isValid;
  final bool autoValidate;
  final TextInputType? textInputKey;
  final TextEditingController? controller;
  final InputBorder? border;
  final InputBorder? enableBorder;
  final InputBorder? focusBorder;
  final InputBorder? errorBorder;
  final TextStyle? hintStyle;
  final TextStyle? errorStyle;
  final TextStyle? suffixStyle;
  final TextStyle? prefixStyle;
  final TextStyle? helperStyle;
  final TextStyle? style;
  final EdgeInsetsGeometry? contentPadding;

  @override
  Widget build(BuildContext context) {
    // media sizes
    final size = MediaQuery.of(context).size;

    sizeFromParent ?? preferSize;
    final postIWidth = realWidth(
        maxWidth: maxIHeight,
        prefSize: preferSize,
        pWidth: sizeFromParent?.prefWidth);
    final postIHeight = realHeight(
        maxHeight: maxIWidth,
        prefSize: preferSize,
        pHeight: sizeFromParent?.prefHeight);
    connectionI ??= 1.i(mediaSize: size);

    // state
    final isValidated = useState(isValid ?? false);
    final controller = useTextEditingController();
    final suffixIcon = isValidated.value
        ? const Icon(
            Icons.check,
            color: AppColors.inputEnableColor,
          )
        : IconButton(
            onPressed: clearIconPressed ?? controller.clear,
            icon: const Icon(
              Icons.cancel_outlined,
              color: AppColors.inputErrorColor,
            ),
          );
    final node = useFocusNode();
    controller.selection = TextSelection.fromPosition(
        TextPosition(offset: controller.text.trim().length));
    final ValueNotifier<Widget?> suffixIconState = useState(suffixIcon);
    return Container(
      margin: EdgeInsets.all(1.p(i: connectionI!)),
      height: postIHeight * connectionI!,
      width: postIWidth * connectionI!,
      alignment: Alignment.center,
      child: TextFormField(
        keyboardType: textInputKey,
        autovalidate: autoValidate,
        controller: controller,
        onTap: () {
          node.requestFocus();
        },
        focusNode: node,
        initialValue: initialValue,
        inputFormatters: inputFormatters,
        onEditingComplete: onEditCompleteDone
            ? node.unfocus
            : () {
                suffixIconState.value = null;
                node.nextFocus();
                node.nextFocus();
                suffixIconState.value = suffixIcon;
              },
        textInputAction:
            onEditCompleteDone ? TextInputAction.done : TextInputAction.next,
        maxLength: maxLength,
        validator: validator,
        onChanged: onChanged,
        style: style ?? TextStyle(fontSize: 3.sp(connectionI!)),
        decoration: InputDecoration(
          contentPadding:
              contentPadding ?? const EdgeInsets.fromLTRB(5.0, 1.0, 5.0, 1.0),
          labelText: labelText,
          floatingLabelBehavior: FloatingLabelBehavior.always,
          hintText: hintText,
          suffixIconConstraints:
              AppDecorations.inpuTextIconBoxContraints(connectionI!),
          prefixIconConstraints:
              AppDecorations.inpuTextIconBoxContraints(connectionI!),
          suffixIcon:
              autoValidate && postIWidth > 2 ? suffixIconState.value : null,
          prefixIcon: postIWidth > 1 ? prefixIcon : null,

          hintStyle: hintStyle ?? TextStyle(fontSize: 2.sp(connectionI!)),
          errorStyle: errorStyle ?? TextStyle(fontSize: 2.sp(connectionI!)),
          labelStyle: labelStyle ?? TextStyle(fontSize: 2.sp(connectionI!)),
          suffixStyle: suffixStyle ?? TextStyle(fontSize: 2.sp(connectionI!)),
          prefixStyle: prefixStyle ?? TextStyle(fontSize: 2.sp(connectionI!)),
          helperStyle: helperStyle ?? TextStyle(fontSize: 2.sp(connectionI!)),
          // counterText: '15',
          border: border ?? AppDecorations.inputBorder,
          enabledBorder: enableBorder ?? AppDecorations.enableBorderColor,
          errorBorder: errorBorder ?? AppDecorations.errorBorderColor,
          focusedBorder: focusBorder ?? AppDecorations.focusBorderColor,
        ),
      ),
    );
  }
}
